/**
 * Copyright (c) 2000-2013 Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.cache.transactional;

import com.liferay.portal.kernel.cache.PortalCache;
import com.liferay.portal.kernel.util.InitialThreadLocal;
import com.liferay.portal.util.PropsValues;

import java.io.Serializable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Shuyang Zhou
 */
public class TransactionalPortalCacheHelper {

	public static void begin() {
		if (!PropsValues.TRANSACTIONAL_CACHE_ENABLED) {
			return;
		}

		_pushPortalCacheMap();
	}

	public static void commit() {
		if (!PropsValues.TRANSACTIONAL_CACHE_ENABLED) {
			return;
		}

		Map<PortalCache, UncommittedBuffer> portalCacheMap =
			_popPortalCacheMap();

		for (Map.Entry<PortalCache, UncommittedBuffer>
				portalCacheMapEntry : portalCacheMap.entrySet()) {

			PortalCache portalCache = portalCacheMapEntry.getKey();

			UncommittedBuffer uncommittedBuffer =
				portalCacheMapEntry.getValue();

			uncommittedBuffer.commitTo(portalCache);
		}

		portalCacheMap.clear();
	}

	public static boolean isEnabled() {
		if (!PropsValues.TRANSACTIONAL_CACHE_ENABLED) {
			return false;
		}

		List<Map<PortalCache, UncommittedBuffer>> portalCacheList =
			_portalCacheListThreadLocal.get();

		return !portalCacheList.isEmpty();
	}

	public static void rollback() {
		if (!PropsValues.TRANSACTIONAL_CACHE_ENABLED) {
			return;
		}

		Map<PortalCache, UncommittedBuffer> portalCacheMap =
			_popPortalCacheMap();

		portalCacheMap.clear();
	}

	protected static Object get(PortalCache portalCache, Serializable key) {
		Map<PortalCache, UncommittedBuffer> portalCacheMap =
			_peekPortalCacheMap();

		UncommittedBuffer uncommittedBuffer = portalCacheMap.get(portalCache);

		if (uncommittedBuffer == null) {
			return null;
		}

		Object object = uncommittedBuffer.get(key);

		if (object instanceof TTLValue) {
			TTLValue ttLValue = (TTLValue)object;

			object = ttLValue._value;
		}

		return object;
	}

	protected static void put(
		PortalCache portalCache, Serializable key, Object value) {

		Map<PortalCache, UncommittedBuffer> portalCacheMap =
			_peekPortalCacheMap();

		UncommittedBuffer uncommittedBuffer = portalCacheMap.get(portalCache);

		if (uncommittedBuffer == null) {
			uncommittedBuffer = new UncommittedBuffer();

			portalCacheMap.put(portalCache, uncommittedBuffer);
		}

		uncommittedBuffer.put(key, value);
	}

	protected static void put(
		PortalCache portalCache, Serializable key, Object value, int ttl) {

		Map<PortalCache, UncommittedBuffer> portalCacheMap =
			_peekPortalCacheMap();

		UncommittedBuffer uncommittedBuffer = portalCacheMap.get(portalCache);

		if (uncommittedBuffer == null) {
			uncommittedBuffer = new UncommittedBuffer();

			portalCacheMap.put(portalCache, uncommittedBuffer);
		}

		uncommittedBuffer.put(key, new TTLValue(ttl, value));
	}

	protected static void removeAll(PortalCache portalCache) {
		Map<PortalCache, UncommittedBuffer> portalCacheMap =
			_peekPortalCacheMap();

		UncommittedBuffer uncommittedBuffer = portalCacheMap.get(portalCache);

		if (uncommittedBuffer == null) {
			uncommittedBuffer = new UncommittedBuffer();

			portalCacheMap.put(portalCache, uncommittedBuffer);
		}

		uncommittedBuffer.removeAll();
	}

	private static Map<PortalCache, UncommittedBuffer>
		_peekPortalCacheMap() {

		List<Map<PortalCache, UncommittedBuffer>> portalCacheList =
			_portalCacheListThreadLocal.get();

		return portalCacheList.get(portalCacheList.size() - 1);
	}

	private static Map<PortalCache, UncommittedBuffer>
		_popPortalCacheMap() {

		List<Map<PortalCache, UncommittedBuffer>> portalCacheList =
			_portalCacheListThreadLocal.get();

		return portalCacheList.remove(portalCacheList.size() - 1);
	}

	private static void _pushPortalCacheMap() {
		List<Map<PortalCache, UncommittedBuffer>> portalCacheList =
			_portalCacheListThreadLocal.get();

		portalCacheList.add(new HashMap<PortalCache, UncommittedBuffer>());
	}

	private static ThreadLocal<List<Map<PortalCache, UncommittedBuffer>>>
		_portalCacheListThreadLocal = new InitialThreadLocal
			<List<Map<PortalCache, UncommittedBuffer>>>(
				TransactionalPortalCacheHelper.class.getName() +
					"._portalCacheListThreadLocal",
				new ArrayList<Map<PortalCache, UncommittedBuffer>>());

	private static class TTLValue {

		public TTLValue(int ttl, Object value) {
			_ttl = ttl;
			_value = value;
		}

		private int _ttl;
		private Object _value;

	}

	private static class UncommittedBuffer {

		public void commitTo(PortalCache portalCache) {
			if (_removeAll) {
				portalCache.removeAll();
			}

			for (Map.Entry<? extends Serializable, List<Object>> entry :
					_uncommittedMap.entrySet()) {

				Serializable key = entry.getKey();
				List<Object> valueList = entry.getValue();

				for (Object value : valueList) {
					if (value == TransactionalPortalCache.NULL_HOLDER) {
						portalCache.remove(key);
					}
					else if (value instanceof TTLValue) {
						TTLValue ttlValue = (TTLValue)value;

						portalCache.put(key, ttlValue._value, ttlValue._ttl);
					}
					else {
						portalCache.put(key, value);
					}
				}
			}
		}

		public Object get(Serializable key) {
			List<Object> valueList = _uncommittedMap.get(key);

			Object value = null;

			if (valueList != null) {
				value = valueList.get(valueList.size() - 1);
			}

			if ((value == null) && _removeAll) {
				value = TransactionalPortalCache.NULL_HOLDER;
			}

			return value;
		}

		public void put(Serializable key, Object value) {
			List<Object> valueList = _uncommittedMap.get(key);

			if (valueList == null) {
				valueList = new ArrayList<Object>();
			}

			valueList.add(value);

			_uncommittedMap.put(key, valueList);
		}

		public void removeAll() {
			_uncommittedMap.clear();

			_removeAll = true;
		}

		private boolean _removeAll;
		private Map<Serializable, List<Object>> _uncommittedMap =
			new HashMap<Serializable, List<Object>>();

	}

}