/**
 * Copyright (c) 2000-2013 Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.service.impl;

import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.Digester;
import com.liferay.portal.kernel.util.DigesterUtil;
import com.liferay.portal.kernel.util.FileUtil;
import com.liferay.portal.kernel.util.StreamUtil;
import com.liferay.portal.kernel.util.StringBundler;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portal.model.CompanyConstants;
import com.liferay.portal.model.Group;
import com.liferay.portal.model.User;
import com.liferay.portal.service.base.StagingLocalServiceBaseImpl;
import com.liferay.portlet.documentlibrary.NoSuchFileException;
import com.liferay.portlet.documentlibrary.model.DLFolderConstants;
import com.liferay.portlet.documentlibrary.store.DLStoreUtil;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * @author Michael C. Han
 * @author Mate Thurzo
 */
public class StagingLocalServiceImpl extends StagingLocalServiceBaseImpl {

	public void cleanUpStagingRequest(long groupId, long stagingRequestId)
		throws PortalException, SystemException {

		long repositoryId = CompanyConstants.SYSTEM;

		Group group = groupLocalService.getGroup(groupId);
		long companyId = group.getCompanyId();

		String directoryName = getStagingRequestDirectory(
			companyId, stagingRequestId);

		if (DLStoreUtil.hasDirectory(companyId, repositoryId, directoryName)) {
			DLStoreUtil.deleteDirectory(companyId, repositoryId, directoryName);
		}
	}

	public long createStagingRequest(long userId, long groupId, String checksum)
		throws PortalException, SystemException {

		User user = userLocalService.getUser(userId);

		long companyId = user.getCompanyId();

		long stagingRequestId = System.nanoTime();

		String directoryName = getStagingRequestDirectory(
			companyId, stagingRequestId);

		if (!DLStoreUtil.hasDirectory(
				companyId, DLFolderConstants.DEFAULT_PARENT_FOLDER_ID,
				directoryName)) {

			DLStoreUtil.addDirectory(
				companyId, DLFolderConstants.DEFAULT_PARENT_FOLDER_ID,
				directoryName);
		}

		DLStoreUtil.addFile(
			companyId, DLFolderConstants.DEFAULT_PARENT_FOLDER_ID,
			directoryName + StringPool.SLASH + _CHECK_SUM_FILE_NAME,
			checksum.getBytes());

		DLStoreUtil.addFile(
			companyId, DLFolderConstants.DEFAULT_PARENT_FOLDER_ID,
			directoryName + StringPool.SLASH + _TARGET_GROUP_FILE_NAME,
			String.valueOf(groupId).getBytes());

		return stagingRequestId;
	}

	public void publishStagingRequest(
			long userId, long stagingRequestId, boolean privateLayout,
			Map<String, String[]> parameterMap)
		throws PortalException, SystemException {

		User user = userLocalService.getUser(userId);

		long companyId = user.getCompanyId();

		String directoryName = getStagingRequestDirectory(
			companyId, stagingRequestId);

		byte[] bytes = DLStoreUtil.getFileAsBytes(
			companyId, DLFolderConstants.DEFAULT_PARENT_FOLDER_ID,
			directoryName + StringPool.SLASH + _TARGET_GROUP_FILE_NAME);

		String groupIdString = new String(bytes);

		long groupId = Long.parseLong(groupIdString);

		File stagingRequestFile = null;

		try {
			stagingRequestFile = getStagingRequestFileEntry(
				userId, stagingRequestId);

			layoutLocalService.importLayouts(
				userId, groupId, privateLayout, parameterMap,
				stagingRequestFile);
		}
		finally {
			if (stagingRequestFile != null) {
				FileUtil.delete(stagingRequestFile);
			}
		}
	}

	public void updateStagingRequest(
			long userId, long stagingRequestId, String fileName, byte[] bytes)
		throws PortalException, SystemException {

		User user = userLocalService.getUser(userId);

		long companyId = user.getCompanyId();

		String directoryName = getStagingRequestDirectory(
			companyId, stagingRequestId);

		if (fileName.contains(StringPool.PERIOD)) {
			fileName = fileName.substring(
				0, fileName.indexOf(StringPool.PERIOD));
		}

		fileName = fileName.concat(
			StringPool.UNDERLINE).concat(
			String.valueOf(System.currentTimeMillis()));

		DLStoreUtil.addFile(
			companyId, DLFolderConstants.DEFAULT_PARENT_FOLDER_ID,
			directoryName + StringPool.SLASH + fileName, bytes);
	}

	protected File fetchStagingRequestFile(long companyId, String fileName)
		throws PortalException, SystemException {

		long repositoryId = DLFolderConstants.DEFAULT_PARENT_FOLDER_ID;

		try {
			return DLStoreUtil.getFile(companyId, repositoryId, fileName);
		}
		catch (NoSuchFileException nsf) {
			return null;
		}
	}

	protected String getAssembledFileName(long stagingRequestId) {
		return _ASSEMBLED_LAR_PREFIX + String.valueOf(stagingRequestId) +
			".lar";
	}

	protected String getStagingRequestDirectory(
		long companyId, long stagingRequestId) {

		StringBundler stringBundler = new StringBundler(5);

		stringBundler.append(String.valueOf(companyId));
		stringBundler.append(StringPool.SLASH);
		stringBundler.append(
			String.valueOf(DLFolderConstants.DEFAULT_PARENT_FOLDER_ID));
		stringBundler.append(StringPool.SLASH);
		stringBundler.append(stagingRequestId);

		return stringBundler.toString();
	}

	protected File getStagingRequestFileEntry(
			long userId, long stagingRequestId)
		throws PortalException, SystemException {

		User user = userLocalService.getUser(userId);

		long companyId = user.getCompanyId();
		long repositoryId = DLFolderConstants.DEFAULT_PARENT_FOLDER_ID;

		String directoryName = getStagingRequestDirectory(
			companyId, stagingRequestId);

		FileOutputStream fileOutputStream = null;

		File tempFile = null;

		try {
			tempFile = FileUtil.createTempFile("lar");
			fileOutputStream = new FileOutputStream(tempFile);

			String[] fileNames = DLStoreUtil.getFileNames(
				companyId, repositoryId, directoryName);

			List<String> fileNamesList = new ArrayList<String>(
				Arrays.asList(fileNames));

			Iterator<String> fileNamesIterator = fileNamesList.iterator();

			while (fileNamesIterator.hasNext()) {
				String fileName = fileNamesIterator.next();

				if (!fileName.contains(StringPool.UNDERLINE)) {
					fileNamesIterator.remove();
				}
			}

			Collections.sort(fileNamesList, new Comparator<String>() {
				public int compare(String value1, String value2) {
					String[] value1Split = StringUtil.split(
						value1, StringPool.UNDERLINE);

					long value1Timestamp = Long.parseLong(value1Split[1]);

					String[] value2Split = StringUtil.split(
						value2, StringPool.UNDERLINE);

					long value2Timestamp = Long.parseLong(value2Split[1]);

					if (value1Timestamp < value2Timestamp) {
						return -1;
					}
					else if (value1Timestamp > value2Timestamp) {
						return 1;
					}

					return 0;
				}
			});

			for (String fileName : fileNamesList) {
				InputStream inputStream = DLStoreUtil.getFileAsStream(
					companyId, repositoryId, fileName);

				try {
					StreamUtil.transfer(inputStream, fileOutputStream, false);
				}
				finally {
					StreamUtil.cleanUp(inputStream);
				}
			}

			FileInputStream fileInputStream = new FileInputStream(tempFile);

			String checksum = DigesterUtil.digestHex(
				Digester.MD5, fileInputStream);

			InputStream checksumInputStream = DLStoreUtil.getFileAsStream(
				companyId, repositoryId,
				directoryName + StringPool.SLASH + _CHECK_SUM_FILE_NAME);

			File checksumFile = FileUtil.createTempFile(checksumInputStream);

			if (!FileUtil.isSameContent(checksumFile, checksum)) {
				throw new SystemException("Invalid checksum for LAR file");
			}

			return tempFile;

		}
		catch (IOException ioe) {
			throw new SystemException("Unable to reassemble LAR file", ioe);
		}
		finally {
			StreamUtil.cleanUp(fileOutputStream);
		}
	}

	private static final String _ASSEMBLED_LAR_PREFIX = "assembled_";

	private static final String _CHECK_SUM_FILE_NAME = "checksum.txt";

	private static final String _TARGET_GROUP_FILE_NAME = "targetGroup.txt";

	private static Log _log = LogFactoryUtil.getLog(
		StagingLocalServiceImpl.class);

}
