/**
 * Copyright (c) 2000-2013 Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portlet.dynamicdatamapping.util;

import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.json.JSONFactoryUtil;
import com.liferay.portal.kernel.json.JSONObject;
import com.liferay.portal.kernel.security.pacl.DoPrivileged;
import com.liferay.portal.kernel.servlet.ServletResponseUtil;
import com.liferay.portal.kernel.upload.UploadRequest;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.MimeTypesUtil;
import com.liferay.portal.kernel.util.StreamUtil;
import com.liferay.portal.kernel.util.StringBundler;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.BaseModel;
import com.liferay.portal.model.CompanyConstants;
import com.liferay.portal.service.ServiceContext;
import com.liferay.portal.util.PortalUtil;
import com.liferay.portlet.documentlibrary.model.DLFileEntryMetadata;
import com.liferay.portlet.documentlibrary.model.DLFileEntryMetadataModel;
import com.liferay.portlet.documentlibrary.model.DLFileVersion;
import com.liferay.portlet.documentlibrary.store.DLStoreUtil;
import com.liferay.portlet.documentlibrary.store.Store;
import com.liferay.portlet.dynamicdatalists.model.DDLRecord;
import com.liferay.portlet.dynamicdatalists.model.DDLRecordModel;
import com.liferay.portlet.dynamicdatalists.model.DDLRecordVersion;
import com.liferay.portlet.dynamicdatamapping.NoSuchTemplateException;
import com.liferay.portlet.dynamicdatamapping.model.DDMStructure;
import com.liferay.portlet.dynamicdatamapping.model.DDMTemplate;
import com.liferay.portlet.dynamicdatamapping.service.DDMStructureLocalServiceUtil;
import com.liferay.portlet.dynamicdatamapping.service.DDMTemplateLocalServiceUtil;
import com.liferay.portlet.dynamicdatamapping.storage.Field;
import com.liferay.portlet.dynamicdatamapping.storage.FieldConstants;
import com.liferay.portlet.dynamicdatamapping.storage.Fields;
import com.liferay.portlet.dynamicdatamapping.storage.StorageEngineUtil;

import java.io.InputStream;
import java.io.Serializable;

import java.util.Date;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author Eduardo Lundgren
 */
@DoPrivileged
public class DDMImpl implements DDM {

	public static final String TYPE_CHECKBOX = "checkbox";

	public static final String TYPE_DDM_DOCUMENTLIBRARY = "ddm-documentlibrary";

	public static final String TYPE_DDM_FILEUPLOAD = "ddm-fileupload";

	public static final String TYPE_RADIO = "radio";

	public static final String TYPE_SELECT = "select";

	@Override
	public Fields getFields(
			long ddmStructureId, long ddmTemplateId,
			ServiceContext serviceContext)
		throws PortalException, SystemException {

		return getFields(
			ddmStructureId, ddmTemplateId, StringPool.BLANK, serviceContext);
	}

	@Override
	public Fields getFields(
			long ddmStructureId, long ddmTemplateId, String fieldNamespace,
			ServiceContext serviceContext)
		throws PortalException, SystemException {

		DDMStructure ddmStructure = DDMStructureLocalServiceUtil.getStructure(
			ddmStructureId);

		try {
			DDMTemplate ddmTemplate = DDMTemplateLocalServiceUtil.getTemplate(
				ddmTemplateId);

			ddmStructure.setXsd(ddmTemplate.getScript());
		}
		catch (NoSuchTemplateException nste) {
		}

		Set<String> fieldNames = ddmStructure.getFieldNames();

		Fields fields = new Fields();

		for (String fieldName : fieldNames) {
			Field field = new Field();

			field.setName(fieldName);

			String fieldDataType = ddmStructure.getFieldDataType(fieldName);
			String fieldType = ddmStructure.getFieldType(fieldName);
			Serializable fieldValue = serviceContext.getAttribute(
				fieldNamespace + fieldName);

			if (fieldDataType.equals(FieldConstants.DATE)) {
				int fieldValueMonth = GetterUtil.getInteger(
					serviceContext.getAttribute(
						fieldNamespace + fieldName + "Month"));
				int fieldValueYear = GetterUtil.getInteger(
					serviceContext.getAttribute(
						fieldNamespace + fieldName + "Year"));
				int fieldValueDay = GetterUtil.getInteger(
					serviceContext.getAttribute(
						fieldNamespace + fieldName + "Day"));

				Date fieldValueDate = PortalUtil.getDate(
					fieldValueMonth, fieldValueDay, fieldValueYear);

				if (fieldValueDate != null) {
					fieldValue = String.valueOf(fieldValueDate.getTime());
				}
			}

			if (Validator.isNull(fieldValue) ||
				fieldDataType.equals(FieldConstants.FILE_UPLOAD)) {

				continue;
			}

			if (fieldType.equals(DDMImpl.TYPE_RADIO) ||
				fieldType.equals(DDMImpl.TYPE_SELECT)) {

				if (fieldValue instanceof String) {
					fieldValue = new String[] {String.valueOf(fieldValue)};
				}

				fieldValue = JSONFactoryUtil.serialize(fieldValue);
			}

			Serializable fieldValueSerializable =
				FieldConstants.getSerializable(
					fieldDataType, GetterUtil.getString(fieldValue));

			field.setValue(fieldValueSerializable);

			fields.put(field);
		}

		return fields;
	}

	@Override
	public Fields getFields(long ddmStructureId, ServiceContext serviceContext)
		throws PortalException, SystemException {

		return getFields(ddmStructureId, 0, serviceContext);
	}

	@Override
	public Fields getFields(
			long ddmStructureId, String fieldNamespace,
			ServiceContext serviceContext)
		throws PortalException, SystemException {

		return getFields(ddmStructureId, 0, fieldNamespace, serviceContext);
	}

	@Override
	public String getFileUploadPath(BaseModel<?> baseModel) {
		StringBundler sb = new StringBundler(7);

		try {
			long primaryKey = 0;

			String version = StringPool.BLANK;

			if (baseModel instanceof DDLRecordModel) {
				DDLRecord record = (DDLRecord)baseModel;

				primaryKey = record.getPrimaryKey();

				DDLRecordVersion recordVersion =
					record.getLatestRecordVersion();

				version = recordVersion.getVersion();
			}
			else if (baseModel instanceof DLFileEntryMetadataModel) {
				DLFileEntryMetadata fileEntryMetadata =
					(DLFileEntryMetadata)baseModel;

				primaryKey = fileEntryMetadata.getPrimaryKey();

				DLFileVersion fileVersion = fileEntryMetadata.getFileVersion();

				version = fileVersion.getVersion();
			}

			sb.append("ddm");
			sb.append(StringPool.SLASH);
			sb.append(baseModel.getModelClassName());
			sb.append(StringPool.SLASH);
			sb.append(primaryKey);
			sb.append(StringPool.SLASH);
			sb.append(version);
		}
		catch (Exception e) {
		}

		return sb.toString();
	}

	@Override
	public void sendFieldFile(
			HttpServletRequest request, HttpServletResponse response,
			Field field)
		throws Exception {

		if (field == null) {
			return;
		}

		DDMStructure structure = field.getDDMStructure();

		Serializable fieldValue = field.getValue();

		JSONObject fileJSONObject = JSONFactoryUtil.createJSONObject(
			String.valueOf(fieldValue));

		String fileName = fileJSONObject.getString("name");
		String filePath = fileJSONObject.getString("path");

		InputStream is = DLStoreUtil.getFileAsStream(
			structure.getCompanyId(), CompanyConstants.SYSTEM, filePath);
		long contentLength = DLStoreUtil.getFileSize(
			structure.getCompanyId(), CompanyConstants.SYSTEM, filePath);
		String contentType = MimeTypesUtil.getContentType(fileName);

		ServletResponseUtil.sendFile(
			request, response, fileName, is, contentLength, contentType);
	}

	@Override
	public String uploadFieldFile(
			long structureId, long storageId, BaseModel<?> baseModel,
			String fieldName, ServiceContext serviceContext)
		throws Exception {

		return uploadFieldFile(
			structureId, storageId, baseModel, fieldName, StringPool.BLANK,
			serviceContext);
	}

	@Override
	public String uploadFieldFile(
			long structureId, long storageId, BaseModel<?> baseModel,
			String fieldName, String fieldNamespace,
			Serializable defaultFieldValue, ServiceContext serviceContext)
		throws Exception {

		HttpServletRequest request = serviceContext.getRequest();

		if (!(request instanceof UploadRequest)) {
			return StringPool.BLANK;
		}

		UploadRequest uploadRequest = (UploadRequest)request;

		String fileName = uploadRequest.getFileName(fieldNamespace + fieldName);

		String fieldValue = StringPool.BLANK;

		InputStream inputStream = null;

		Fields fields = StorageEngineUtil.getFields(storageId);

		try {
			inputStream = uploadRequest.getFileAsStream(
				fieldNamespace + fieldName, true);

			if (Validator.isNull(fileName) && (inputStream == null)) {
				fieldValue = GetterUtil.getString(defaultFieldValue);

				if (Validator.isNotNull(fieldValue)) {
					JSONObject recordFileJSONObject =
						JSONFactoryUtil.createJSONObject(fieldValue);

					fileName = recordFileJSONObject.getString("name");

					inputStream = DLStoreUtil.getFileAsStream(
						serviceContext.getCompanyId(), CompanyConstants.SYSTEM,
						recordFileJSONObject.getString("path"));
				}
			}

			if (inputStream != null) {
				String filePath = storeFieldFile(
					baseModel, fieldName, inputStream, serviceContext);

				JSONObject recordFileJSONObject =
					JSONFactoryUtil.createJSONObject();

				recordFileJSONObject.put("name", fileName);
				recordFileJSONObject.put("path", filePath);
				recordFileJSONObject.put(
					"className", baseModel.getModelClassName());
				recordFileJSONObject.put(
					"classPK", String.valueOf(baseModel.getPrimaryKeyObj()));

				fieldValue = recordFileJSONObject.toString();
			}
			else if (fields.contains(fieldName)) {
				return StringPool.BLANK;
			}

			Field field = new Field(structureId, fieldName, fieldValue);

			fields.put(field);

			StorageEngineUtil.update(storageId, fields, true, serviceContext);
		}
		finally {
			StreamUtil.cleanUp(inputStream);
		}

		return fieldValue;
	}

	@Override
	public String uploadFieldFile(
			long structureId, long storageId, BaseModel<?> baseModel,
			String fieldName, String fieldNamespace,
			ServiceContext serviceContext)
		throws Exception {

		return uploadFieldFile(
			structureId, storageId, baseModel, fieldName, fieldNamespace, null,
			serviceContext);
	}

	protected String storeFieldFile(
			BaseModel<?> baseModel, String fieldName, InputStream inputStream,
			ServiceContext serviceContext)
		throws PortalException, SystemException {

		long companyId = serviceContext.getCompanyId();

		String dirName = getFileUploadPath(baseModel);

		if (!DLStoreUtil.hasDirectory(
				companyId, CompanyConstants.SYSTEM, dirName)) {

			DLStoreUtil.addDirectory(
				companyId, CompanyConstants.SYSTEM, dirName);
		}

		String fileName = dirName + StringPool.SLASH + fieldName;

		if (DLStoreUtil.hasFile(
				companyId, CompanyConstants.SYSTEM, fileName,
				Store.VERSION_DEFAULT)) {

			DLStoreUtil.deleteFile(
				companyId, CompanyConstants.SYSTEM, fileName,
				Store.VERSION_DEFAULT);
		}

		DLStoreUtil.addFile(
			companyId, CompanyConstants.SYSTEM, fileName, false, inputStream);

		return fileName;
	}

}