/**
 * Copyright (c) 2000-2013 Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.kernel.log;

import com.liferay.portal.kernel.io.unsync.UnsyncStringWriter;
import com.liferay.portal.kernel.util.UnsyncPrintWriterPool;
import com.liferay.portal.kernel.util.Validator;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.jsp.JspException;

/**
 * @author Brian Wing Shun Chan
 */
public class LogUtil {

	public static final boolean REMOVE_UNKNOWN_SOURCE = true;

	public static final int STACK_TRACE_LENGTH = 20;

	public static void debug(Log log, Properties props) {
		if (log.isDebugEnabled()) {
			UnsyncStringWriter unsyncStringWriter = new UnsyncStringWriter(
				props.size() + 1);

			props.list(UnsyncPrintWriterPool.borrow(unsyncStringWriter));

			log.debug(unsyncStringWriter.toString());
		}
	}

	public static void log(Log log, JspException jspe) {
		Throwable cause = jspe.getCause();

		if (cause == null) {
			cause = jspe;
		}

		if ((cause != jspe) && (cause instanceof JspException)) {
			log(log, (JspException)cause);
		}
		else if (cause instanceof ServletException) {
			log(log, (ServletException)cause);
		}
		else {
			log(log, cause);
		}
	}	
		
	public static void log(Log log, ServletException se) {
			Throwable cause = se.getRootCause();
			if (cause == null) {
				cause = se;
			}
			
			if (cause instanceof JspException) {
				log(log, (JspException)cause);
			}
			else if ((cause != se) && (cause instanceof ServletException)) {
				log(log, (ServletException)cause);
			}			
			else {
				log(log, cause);
			}
	}
	
	public static void log(Log log, Throwable throwable) {
		log(log, throwable, null);
	}

	public static void log(Log log, Throwable throwable, String message) {
		if (throwable == null) {
			if (Validator.isNotNull(message)) {
				log.error(message);

				return;
			}

			throw new IllegalArgumentException(
				"Throwable or message must be set");
		}

		Throwable causeThrowable = throwable;

		while (causeThrowable.getCause() != null) {
			causeThrowable = causeThrowable.getCause();
		}

		StackTraceElement[] stackTraceElements = causeThrowable.getStackTrace();

		// Make the stack trace more readable by limiting the number of
		// elements.

		if (stackTraceElements.length <= STACK_TRACE_LENGTH) {
			if (Validator.isNotNull(message)) {
				log.error(message, causeThrowable);
			}
			else {
				log.error(causeThrowable);
			}

			return;
		}

		int count = 0;

		List<StackTraceElement> stackTraceElementsList =
			new ArrayList<StackTraceElement>();

		for (StackTraceElement stackTraceElement : stackTraceElements) {

			// Make the stack trace more readable by removing elements that
			// refer to classes with no packages, or starts with a $, or are
			// Spring classes, or are standard reflection classes.

			String className = stackTraceElement.getClassName();

			boolean addElement = true;

			if (REMOVE_UNKNOWN_SOURCE &&
				(stackTraceElement.getLineNumber() < 0)) {

				addElement = false;
			}

			if (className.startsWith("$") ||
				className.startsWith("java.lang.reflect.") ||
				className.startsWith("org.springframework.") ||
				className.startsWith("sun.reflect.")) {

				addElement = false;
			}

			if (addElement) {
				stackTraceElementsList.add(stackTraceElement);

				count++;
			}

			if (count >= STACK_TRACE_LENGTH) {
				break;
			}
		}

		stackTraceElements = stackTraceElementsList.toArray(
			new StackTraceElement[stackTraceElementsList.size()]);

		causeThrowable.setStackTrace(stackTraceElements);

		if (Validator.isNotNull(message)) {
			log.error(message, causeThrowable);
		}
		else {
			log.error(causeThrowable);
		}
	}

}